package com.tiangua.star.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tiangua.star.enmus.CommercialCallBackReadyEnum;
import com.tiangua.star.model.CustomerFollowup;
import com.tiangua.star.model.UserStarFollowCallbackParam;
import com.tiangua.star.model.XdOrderDto;
import com.tiangua.star.model.XinLuUserStarCallbackParam;
import com.tiangua.star.service.CallBackService;
import com.tiangua.star.service.XinLuService;
import com.tiangua.star.util.HttpClientThreeUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@Slf4j
public class CasllBackDataProcessor implements CallBackService {

    // HTTP请求工具
    private static final CloseableHttpClient httpClient = HttpClients.createDefault();


    @Value("${md5.callback.url}")
    private String url;

    /**
     * 处理机构数据入口
     */
    @Override
    public void processInstitutionData(List<XdOrderDto> xdOrderDtos) {
        Map<String, List<String>> groupedByBusiId = xdOrderDtos.stream()
                .collect(Collectors.groupingBy(
                        XdOrderDto::getProductBizId,
                        Collectors.mapping(XdOrderDto::getMobileMd5, Collectors.toList())
                ));
        Map<String, List<String>> callBackMap = groupedByBusiId.entrySet().stream()
                .filter(entry -> CommercialCallBackReadyEnum.of(entry.getKey()) != null)
                .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
        log.info("已经对接md5星级回传的机构id:{}", callBackMap.keySet());
        if (CollectionUtils.isEmpty(callBackMap.keySet())) {
            log.info("MD5 无需要处理的星级回传:{}", callBackMap.keySet());
            return;
        }

        for (String busiId : callBackMap.keySet()) {
            CommercialCallBackReadyEnum commercialCallBackReadyEnum = CommercialCallBackReadyEnum.of(busiId);
            if (commercialCallBackReadyEnum != null) {
                switch (commercialCallBackReadyEnum) {
                    case Commercial_ONE:
                        log.info("开始处理机构id:{}", busiId);
                        List<String> md5List = callBackMap.get(busiId);
                        if (CollectionUtils.isEmpty(md5List)) {
                            return;
                        }
                        sendGetRequest(CommercialCallBackReadyEnum.Commercial_ONE.getCode(),
                                CommercialCallBackReadyEnum.Commercial_ONE.getBusid(),commercialCallBackReadyEnum.getUrl(),md5List);
                        break;
//                    case Commercial_Test:
//                        log.info("开始处理机构id:{}", busiId);
//                        List<String> md5List2 = callBackMap.get(busiId);
//                        if (CollectionUtils.isEmpty(md5List2)) {
//                            return;
//                        }
//                        sendGetRequest(CommercialCallBackReadyEnum.Commercial_Test.getCode(),
//                                CommercialCallBackReadyEnum.Commercial_Test.getBusid(),commercialCallBackReadyEnum.getUrl(),md5List2);
//                        break;
                }
            }
        }




    }

    private void sendGetRequest(String productBizId,String busiId,String url,List<String> md5s){
        List<CustomerFollowup> list = new ArrayList<>();
        JSONObject map = new JSONObject();
        JSONObject response = null;
        for (String md5 : md5s) {
            map.put("customer_phone", md5);
            map.put("customer_source", "LCD-HX");
            String responseStr = HttpClientThreeUtil.post(url, map.toJSONString());
            response = JSON.parseObject(responseStr);
            if (response.getInteger("code") == 1) {
                JSONObject data = response.getJSONObject("data");
                CustomerFollowup param = new CustomerFollowup();

                JSONArray logList = data.getJSONArray("log_list");
                if (CollectionUtils.isNotEmpty(logList)) {
                    StringBuilder result = new StringBuilder();
                    for (int i = 0; i < logList.size(); i++) {
                        JSONObject logEntry = logList.getJSONObject(i);
                        String logTime = logEntry.getString("log_time");
                        String content = logEntry.getString("content");
                        result.append(logTime).append("-").append(content);
                        if (i < logList.size() - 1) {
                            result.append(";");
                        }
                    }
                    param.setFollowRemark(result.toString());
                }
                param.setPhoneMd5(md5);
                param.setStarLevel(data.getBigDecimal("customer_starlevel"));
                param.setProductBizId(Integer.parseInt(productBizId));
                param.setBusiId(busiId);
                param.setSourceType(3);
                list.add(param);
            } else {
                log.warn("url:{} ,productBizId:{},md5:{} 获取结果失败",url,productBizId,md5);
            }
        }
        log.info("busiId:{} 处理数据条:{}", busiId, list.size());
        handleMappedData(list);
    }



    private void handleMappedData(List<CustomerFollowup> data) {
        log.info("【CasllBackDataProcessor】成功处理 {} 条映射数据 明细：{}", data.size(), data);
            String post = HttpClientThreeUtil.post(url, JSON.toJSONString(data));
        log.info("【CasllBackDataProcessor】post结果：{}", post);
    }



}
